/*

    File: lang.c

    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include "types.h"
#include "common.h"
#include "lang.h"

struct systypes {
          const unsigned char *name;
};

static const struct systypes *get_sys_types(const part_sys_type_t part_sys);

static const struct systypes i386_sys_types[] = {
  {"\x00" "empty"},
  {"\x01" "FAT12"},
  {"\x02" "XENIX root"},
  {"\x03" "XENIX /usr"},
  {"\x04" "FAT16 <32M"},
  {"\x05" "extended"},
  {"\x06" "FAT16 >32M"},
  {"\x07" "HPFS - NTFS"},
  {"\x09" "AIX data"},
  {"\x0a" "OS/2 Boot Manager"},
  {"\x0b" "FAT32"},
  {"\x0c" "FAT32 LBA"},
  {"\x0E" "FAT16 LBA"},
  {"\x0F" "extended LBA"},
  {"\x10" "OPUS"},
  {"\x11" "hid. FAT12"},
  {"\x12" "Compaq Diagnostics"},
  {"\x14" "hid. FAT16 <32M"},
  {"\x16" "hid. FAT16 >32M"},
  {"\x17" "hid. HPFS/NTFS"},
  {"\x18" "AST swap"},
  {"\x19" "Willowtech Photon"},
  {"\x1b" "hid. FAT32"},
  {"\x1c" "hid. FAT32 LBA"},
  {"\x1e" "hid. FAT16 LBA"},
  {"\x20" "Willowsoft OFS1"},
  {"\x24" "NEC MS-DOS 3.x"},
  {"\x38" "Theos"},
  {"\x3c" "PMagic recovery"},
  {"\x40" "VENIX 80286"},
  {"\x41" "PPC PReP Boot"},
  {"\x42" "W2K Dynamic/SFS"},
  {"\x50" "OnTrack DM RO"},
  {"\x51" "OnTrack DM RW-NOVEL"},
  {"\x52" "CP/M-Microport V/386"},
  {"\x53" "OnTrack DM WO ???"},
  {"\x54" "OnTrack DM DDO"},
  {"\x55" "EZ-Drive"},
  {"\x56" "GoldenBow VFeature"},
  {"\x61" "SpeedStor"},
  {"\x63" "Unixware, HURD, SCO"},
  {"\x64" "NetWare 286"},
  {"\x65" "NetWare 3.11+"},
  {"\x67" "Novell"},
  {"\x68" "Novell"},
  {"\x69" "Novell"},
  {"\x70" "DiskSecure MB"},
  {"\x75" "PC/IX"},
  {"\x80" "Minix v1.1-1.4a"},
  {"\x81" "Minix / old Linux"},
  {"\x82" "Linux Swap"},
  {"\x83" "Linux"},
  {"\x85" "Linux extended"},
  {"\x86" "NT FAT16 V/S set"},
  {"\x87" "HPFS FT mirror-V/S set"},
  {"\x8e" "Linux LVM"},
  {"\x93" "Amoeba"},
  {"\x94" "Amoeba bad block"},
  {"\xa0" "NoteBIOS save2disk"},
  {"\xa5" "FreeBSD"},
  {"\xa6" "OpenBSD"},
  {"\xa8" "Darwin UFS"},
  {"\xa9" "NetBSD"},
  {"\xab" "Darwin boot"},
  {"\xb7" "BSDI"},
  {"\xb8" "BSDI swap"},
  {"\xbe" "Solaris boot"},
  {"\xc1" "secured FAT12"},
  {"\xc4" "secured FAT16"},
  {"\xc6" "sec. Huge-bad FAT16"},
  {"\xc7" "Syrinx Boot-bad NTFS"},
  {"\xd8" "CP/M-86"},
  {"\xdb" "CP/M"},
  {"\xde" "Dell Utility"},
  {"\xe1" "SpeedStor FAT12 ext"},
  {"\xe3" "DOS RO"},
  {"\xe4" "SpeedStor FAT16 ext"},
  {"\xeb" "BeFS"},
  {"\xee" "EFI GPT"},          /* Intel EFI GUID Partition Table */
  {"\xef" "EFI (FAT-12/16/32)"},/* Intel EFI System Partition */
  {"\xf0" "Linux/PA-RISC boot"},/* Linux/PA-RISC boot loader */
  {"\xf1" "Storage Dimensions"},
  {"\xf2" "DOS secondary"},
  {"\xf4" "SpeedStor"},
  {"\xfd" "Linux RAID"},
  {"\xfe" "LANstep"},
  {"\xff" "Xenix bad block"},
  { NULL }
};

static const struct systypes xbsd_fstypes[] = {
/* BSD_FS_UNUSED  */   {"\x00" "unused"},
/* BSD_FS_SWAP    */   {"\x01" "swap"},
/* BSD_FS_V6      */   {"\x02" "Version 6"},
/* BSD_FS_V7      */   {"\x03" "Version 7"},
/* BSD_FS_SYSV    */   {"\x04" "System V"},
/* BSD_FS_V71K    */   {"\x05" "4.1BSD"},
/* BSD_FS_V8      */   {"\x06" "Eighth Edition"},
/* BSD_FS_BSDFFS  */   {"\x07" "4.2BSD"},
#ifdef __alpha__
/* BSD_FS_EXT2    */   {"\x08" "ext2"},
#else
/* BSD_FS_MSDOS   */   {"\x08" "MS-DOS"},
#endif
/* BSD_FS_BSDLFS  */   {"\x09" "4.4LFS"},
/* BSD_FS_OTHER   */   {"\x0a" "unknown"},
/* BSD_FS_HPFS    */   {"\x0b" "HPFS"},
/* BSD_FS_ISO9660 */   {"\x0c" "ISO-9660"},
/* BSD_FS_BOOT    */   {"\x0d" "boot"},
/* BSD_FS_ADOS    */   {"\x0e" "ADOS"},
/* BSD_FS_HFS     */   {"\x0f" "HFS"},
/* BSD_FS_ADVFS   */   {"\x10" "AdvFS"},
                       { NULL }
};

static const struct systypes sgi_sys_types[] = {
  /* SGI_VOLHDR   */  {"\x00" "SGI volhdr"   },
  /* 0x01         */  {"\x01" "SGI trkrepl"  },
  /* 0x02         */  {"\x02" "SGI secrepl"  },
  /* SGI_SWAP     */  {"\x03" "SGI raw"      },
  /* 0x04         */  {"\x04" "SGI bsd"      },
  /* 0x05         */  {"\x05" "SGI sysv"     },
  /* ENTIRE_DISK  */  {"\x06" "SGI volume"   },
  /* SGI_EFS      */  {"\x07" "SGI efs"      },
  /* 0x08         */  {"\x08" "SGI lvol"     },
  /* 0x09         */  {"\x09" "SGI rlvol"    },
  /* SGI_XFS      */  {"\x0a" "SGI xfs"      },
  /* SGI_XFSLOG   */  {"\x0b" "SGI xfslog"   },
  /* SGI_XLV      */  {"\x0c" "SGI xlv"      },
  /* SGI_XVM      */  {"\x0d" "SGI xvm"      },
  /* LINUX_SWAP   */  {"\x82" "Linux swap"   },
  /* LINUX_NATIVE */  {"\x83" "Linux native" },
  /* LINUX_LVM    */  {"\x8d" "Linux LVM"    },
  /* LINUX_RAID   */  {"\xfd" "Linux RAID"   },
                      { NULL             }
};

static const struct systypes sun_sys_types[] = {
  /* 0            */  {"\x00" "Empty"        },
  /* 1            */  {"\x01" "Boot"         },
  /* 2            */  {"\x02" "SunOS root"   },
  /* SUNOS_SWAP   */  {"\x03" "SunOS swap"   },
  /* 4            */  {"\x04" "SunOS usr"    },
  /* WHOLE_DISK   */  {"\x05" "Whole disk"   },
  /* 6            */  {"\x06" "SunOS stand"  },
  /* 7            */  {"\x07" "SunOS var"    },
  /* 8            */  {"\x08" "SunOS home"   },
  /* LINUX_SWAP   */  {"\x82" "Linux swap"   },
  /* LINUX_NATIVE */  {"\x83" "Linux native" },
  /* 0x8e         */  {"\x8e" "Linux LVM"    },
  /* New (2.2.x) raid partition with autodetect using persistent superblock */
  /* 0xfd         */  {"\xfd" "Linux raid autodetect" },
                      { NULL }
};

static const struct systypes *get_sys_types(const part_sys_type_t part_sys)
{
  return (
      part_sys==PART_SYS_SUN ? sun_sys_types :
      part_sys==PART_SYS_SGI ? sgi_sys_types :
      i386_sys_types);
}


const char *partition_type(const unsigned char type, const part_sys_type_t part_sys)
{
  int i;
  const struct systypes *types = get_sys_types(part_sys);
  for (i=0; types[i].name; i++)
    if (types[i].name[0] == type)
      return types[i].name + 1;

  return NULL;
}
